      SUBROUTINE MPX_Init(HANDLER, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
      PARAMETER(NTIMES=10000)
C
C Initialization of MPX processes
C
      INTEGER HANDLER
      INTEGER IERROR
      LOGICAL FLAG
C
      CALL MPI_INITIALIZED(FLAG, IERROR)
      IF(FLAG) RETURN
C
      CALL MPI_INIT(IERROR)
C
      MPX_INTEGER = MPI_INTEGER
      MPX_REAL = MPI_REAL
      MPX_DOUBLE_PRECISION = MPI_DOUBLE_PRECISION
      MPX_COMPLEX = MPI_COMPLEX
      MPX_LOGICAL = MPI_LOGICAL
      MPX_CHARACTER = MPI_CHARACTER
      MPX_COMM_WORLD = MPI_COMM_WORLD
      MPX_MIN = MPI_MIN
      MPX_MAX = MPI_MAX
      MPX_SUM = MPI_SUM
      MPX_PROD = MPI_PROD
      MPX_ANY_SOURCE = MPI_ANY_SOURCE
      MPX_ANY_TAG = MPI_ANY_TAG
      MPX_SUCCESS = MPI_SUCCESS
      MPX_COMM_NULL = MPI_COMM_NULL
      MPX_UNDEFINED = MPI_UNDEFINED
      MPX_DATATYPE_NULL = MPI_DATATYPE_NULL
      MPX_PROC_NULL = MPI_PROC_NULL
      MPX_REQUEST_NULL = MPI_REQUEST_NULL
C
      IF(HANDLER .EQ. MPX_ERRORS_ARE_FATAL) MPX_ERROR_HANDLER =
     *                                     MPI_ERRORS_ARE_FATAL
      IF(HANDLER .EQ. MPX_ERRORS_RETURN) MPX_ERROR_HANDLER =
     *                                  MPI_ERRORS_RETURN
      IF(HANDLER .EQ. MPX_ERRHANDLER) THEN
         CALL MPI_ERRHANDLER_CREATE(MPX_EHANDLER, MPX_ERROR_HANDLER,
     *                              IERROR)
      END IF
      CALL MPI_ERRHANDLER_SET(MPX_COMM_WORLD, MPX_ERROR_HANDLER,
     *                        IERROR)
      MPX_TIME = 0
      MPX_LENGTH = 0
      MPX_COUNT = 0
C
      CALL MPX_BARRIER(MPX_COMM_WORLD, IERROR)
      MPX_DT = 0
      DO I = 1, NTIMES
         T1 = MPX_WTIME()
         T2 = MPX_WTIME()
         MPX_DT = MPX_DT + T2 - T1
      END DO
      MPX_DT = MPX_DT / NTIMES
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Finalize(IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Finalization of MPX processes
C
      INTEGER IERROR
C
      IF(MPX_ERROR_HANDLER .NE. MPI_ERRORS_ARE_FATAL .AND.
     *   MPX_ERROR_HANDLER .NE. MPI_ERRORS_RETURN)
     *   CALL MPI_ERRHANDLER_FREE(MPX_ERROR_HANDLER, IERROR)
      CALL MPI_FINALIZE(IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      FUNCTION MPX_NProc(COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns the number of processes
C
      INTEGER COMM
      INTEGER IERROR
      INTEGER ISIZE
C
      CALL MPI_COMM_SIZE(COMM, ISIZE, IERROR)
      MPX_NProc = ISIZE
C
      RETURN
      END
C-----------------------------------------------------------------------
      FUNCTION MPX_Myid(COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns the identifier of process
C
      INTEGER COMM
      INTEGER IERROR
      INTEGER IRANK
C
      CALL MPI_COMM_RANK(COMM, IRANK, IERROR)
      MPX_Myid = IRANK
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Send(BUF, COUNT, DATATYPE, DEST, MSGTAG, COMM,
     *                    IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Blocking send
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, DEST, MSGTAG, COMM
      INTEGER IERROR
C
      T1 = MPX_WTime()
      CALL MPI_SEND(BUF, COUNT, DATATYPE, DEST, MSGTAG, COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      MPX_LENGTH = MPX_LENGTH + COUNT + 1
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Recv(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *                    COMM, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Blocking receive
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, SOURCE, MSGTAG,
     *        STATUS(MPX_STATUS_SIZE), COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_RECV(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *              COMM, STATUS, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Get_Count(STATUS, DATATYPE, COUNT, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns the number of received data
C
      INTEGER STATUS(MPX_STATUS_SIZE), DATATYPE
      INTEGER IERROR
      INTEGER COUNT
C
      CALL MPI_GET_COUNT(STATUS, DATATYPE, COUNT, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Probe(SOURCE, MSGTAG, COMM, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Get the information about message
C
      INTEGER SOURCE, MSGTAG, STATUS(MPX_STATUS_SIZE), COMM
      INTEGER IERROR
C
      CALL MPI_PROBE(SOURCE, MSGTAG, COMM, STATUS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_ISend(BUF, COUNT, DATATYPE, DEST, MSGTAG, COMM,
     *                     REQUEST, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Nonblocking send
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, DEST, MSGTAG, REQUEST, COMM
      INTEGER IERROR
C
      CALL MPI_ISEND(BUF, COUNT, DATATYPE, DEST, MSGTAG, COMM,
     *               REQUEST, IERROR)
      MPX_LENGTH = MPX_LENGTH + COUNT + 1
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_IRecv(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *                     COMM, REQUEST, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Nonblocking receive
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, SOURCE, MSGTAG, REQUEST, COMM
      INTEGER IERROR
C
      CALL MPI_IRECV(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *               COMM, REQUEST, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Wait(REQUEST, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Wait for completion of MPX_ISend and MPX_IReceive
C
      INTEGER REQUEST, STATUS(MPX_STATUS_SIZE)
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_WAIT(REQUEST, STATUS, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Test(REQUEST, FLAG, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns .true. if send-receive operations are completed
C
      INTEGER REQUEST, STATUS(MPX_STATUS_SIZE)
      LOGICAL FLAG
      INTEGER IERROR
C
      CALL MPI_TEST(REQUEST, FLAG, STATUS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_IProbe(SOURCE, MSGTAG, COMM, FLAG, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns .true. if the message is sent
C
      INTEGER SOURCE, MSGTAG, STATUS(MPX_STATUS_SIZE), COMM
      LOGICAL FLAG
      INTEGER IERROR
C
      CALL MPI_IPROBE(SOURCE, MSGTAG, COMM, FLAG, STATUS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Send_Init(BUF, COUNT, DATATYPE, DEST, MSGTAG,
     *                        COMM, REQUEST, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Initialization of send operation for collective communication
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, DEST, MSGTAG, REQUEST, COMM
      INTEGER IERROR
C
      CALL MPI_SEND_INIT(BUF, COUNT, DATATYPE, DEST, MSGTAG,
     *                   COMM, REQUEST, IERROR)
      MPX_LENGTH = MPX_LENGTH + COUNT + 1
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Recv_Init(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *                           COMM, REQUEST, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Initialization of receive operation for collective communication
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, SOURCE, MSGTAG, REQUEST, COMM
      INTEGER IERROR
C
      CALL MPI_RECV_INIT(BUF, COUNT, DATATYPE, SOURCE, MSGTAG,
     *                   COMM, REQUEST, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_StartAll(COUNT, REQUESTS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Start of collective communication
C
      INTEGER COUNT, REQUESTS(*)
      INTEGER IERROR
C
      CALL MPI_STARTALL(COUNT, REQUESTS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Bcast(BUF, COUNT, DATATYPE, SOURCE, COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Broadcasting of the message
C
      REAL BUF(*)
      INTEGER COUNT, DATATYPE, SOURCE, COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
      INTEGER MYID, NPROC
C
      MYID = MPX_MYID(MPX_COMM_WORLD, IERROR)
      NPROC = MPX_NPROC(MPX_COMM_WORLD, IERROR)
      T1 = MPX_WTime()
      CALL MPI_BCAST(BUF, COUNT, DATATYPE, SOURCE, COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      IF(MYID .EQ. SOURCE) THEN
         MPX_LENGTH = MPX_LENGTH + COUNT + 1
         MPX_COUNT = MPX_COUNT + NPROC - 1
      END IF
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Gather(SBUF, COUNT, DATATYPE, RBUF, DEST, COMM,
     *                      IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Gather of the messages from all processes
C
      REAL SBUF(*), RBUF(*)
      INTEGER COUNT, DATATYPE, DEST, COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
      INTEGER MYID
C
      MYID = MPX_MYID(MPX_COMM_WORLD, IERROR)
      T1 = MPX_WTime()
      CALL MPI_GATHER(SBUF, COUNT, DATATYPE, RBUF, COUNT, DATATYPE,
     *                DEST, COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      IF(MYID .NE. DEST) THEN
         MPX_LENGTH = MPX_LENGTH + COUNT + 1
         MPX_COUNT = MPX_COUNT + 1
      END IF
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Barrier(COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Barrier synchronization
C
      INTEGER COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_BARRIER(COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Sig_Send(DEST, SIGTAG, COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Send of the signal
C
      INTEGER DEST, SIGTAG, COMM
      INTEGER IERROR
C
      CALL MPI_SEND(DEST, 0, MPI_INTEGER, DEST, SIGTAG, COMM, IERROR)
      MPX_LENGTH = MPX_LENGTH + 1
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Sig_Test(SOURCE, SIGTAG, FLAG, COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Test for signal sending
C
      INTEGER SOURCE, SIGTAG, COMM
      LOGICAL FLAG
      INTEGER STATUS(MPX_STATUS_SIZE)
      INTEGER IERROR
C
      CALL MPI_IPROBE(SOURCE, SIGTAG, COMM, FLAG, STATUS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Sig_Wait(SOURCE, SIGTAG, COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Wait for signal
C
      INTEGER SOURCE, SIGTAG, COMM
      INTEGER STATUS(MPX_STATUS_SIZE)
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_RECV(SOURCE, 0, MPI_INTEGER, SOURCE, SIGTAG,
     *              COMM, STATUS, IERROR)
      T2 = MPX_Wtime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      FUNCTION MPX_Cart_Create(NDIMS, DIMS, PERIODS, IDCART, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Creation of multidimential logical structure
C
      INTEGER NDIMS, DIMS(*), IDCART
      LOGICAL PERIODS(*)
      LOGICAL REORDER
      INTEGER IERROR
C
      REORDER = .TRUE.
      CALL MPI_CART_CREATE(MPI_COMM_WORLD, NDIMS, DIMS, PERIODS,
     *                     REORDER, IDCART, IERROR)
      IF(IDCART .EQ. MPI_COMM_NULL) THEN
         MPX_Cart_Create = .FALSE.
      ELSE
         MPX_Cart_Create = .TRUE.
      END IF
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Cart_MyId(IDCART, COORD, MYID, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Detection of identifier of process with coordinates COORD
C
      INTEGER IDCART, COORD(*), MYID
      INTEGER IERROR
C
      CALL MPI_CART_RANK(IDCART, COORD, MYID, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Cart_Coord(IDCART, MYID, COORD, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Detection of coordinates of process with identifier MYID
C
      INTEGER IDCART, MYID, COORD(*)
      INTEGER IERROR, NDIMS
C
      CALL MPI_CARTDIM_GET(IDCART, NDIMS, IERROR)
      CALL MPI_CART_COORDS(IDCART, MYID, NDIMS, COORD, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Abort(COMM, ERRORCODE, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Abort
C
      INTEGER COMM, ERRORCODE
      INTEGER IERROR
C
      CALL MPI_ABORT(COMM, ERRORCODE, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_SendRecv(SBUF, SCOUNT, SDATATYPE, DEST, STAG,
     *                        RBUF, RCOUNT, RDATATYPE, SRC,  RTAG,
     *                        COMM, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Blocking send-receive
C
      REAL SBUF(*), RBUF(*)
      INTEGER SCOUNT, SDATATYPE, DEST, STAG,
     *        RCOUNT, RDATATYPE, SRC,  RTAG,
     *        STATUS(MPX_STATUS_SIZE), COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_SENDRECV(SBUF, SCOUNT, SDATATYPE, DEST, STAG,
     *                  RBUF, RCOUNT, RDATATYPE, SRC,  RTAG,
     *                  COMM, STATUS, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      MPX_LENGTH = MPX_LENGTH + SCOUNT + 1
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      FUNCTION MPX_Wtime()
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns time
C
      MPX_Wtime = MPI_WTIME()
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_WaitAll(COUNT, REQUESTS, STATUSES, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Waits all requested processes
C
      INTEGER COUNT, REQUESTS(*), STATUSES(MPI_STATUS_SIZE, *)
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_WAITALL(COUNT, REQUESTS, STATUSES, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_WaitAny(COUNT, REQUESTS, INDEX, STATUS, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Waits any requested process
C
      INTEGER COUNT, INDEX, REQUESTS(*), STATUS(MPI_STATUS_SIZE)
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_WAITANY(COUNT, REQUESTS, INDEX, STATUS, IERROR)
      T2 = MPX_WTIME()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_WaitSome(ICOUNT, REQUESTS, OCOUNT, INDICES,
     *                        STATUSES, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Waits some requested processes, and returns completed
C
      INTEGER ICOUNT, OCOUNT, INDICES(*), REQUESTS(*),
     *        STATUSES(MPI_STATUS_SIZE, *)
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
C
      T1 = MPX_WTime()
      CALL MPI_WAITSOME(ICOUNT, REQUESTS, OCOUNT,
     *                  INDICES, STATUSES, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_TestAll(COUNT, REQUESTS, FLAG, STATUSES, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Tests all requested processes
C
      INTEGER COUNT, REQUESTS(*), STATUSES(MPI_STATUS_SIZE, *)
      LOGICAL FLAG
      INTEGER IERROR
C
      CALL MPI_TESTALL(COUNT, REQUESTS, FLAG, STATUSES, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_TestAny(COUNT, REQUESTS, INDEX, FLAG, STATUS,
     *                       IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Tests any requested process
C
      INTEGER COUNT, INDEX, REQUESTS(*), STATUS(MPI_STATUS_SIZE)
      LOGICAL FLAG
      INTEGER IERROR
C
      CALL MPI_TESTANY(COUNT, REQUESTS, INDEX, FLAG, STATUS, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_TestSome(ICOUNT, REQUESTS, OCOUNT, INDICES,
     *                        STATUSES, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Tests some requested processes, and returns completed
C
      INTEGER ICOUNT, OCOUNT, INDICES(*), REQUESTS(*),
     *        STATUSES(MPI_STATUS_SIZE, *)
      INTEGER IERROR
C
      CALL MPI_TESTSOME(ICOUNT, REQUESTS, OCOUNT,
     *                  INDICES, STATUSES, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Comm_Free(COMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Sets the handle to MPX_COMM_NULL
C
      INTEGER COMM
      INTEGER IERROR
C
      CALL MPI_COMM_FREE(COMM, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Comm_Split(COMM, COLOR, KEY, NEWCOMM, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Partitions the group COMM into disjoint subgroups one for each
C   value of COLOR
C
      INTEGER COMM, COLOR, KEY, NEWCOMM
      INTEGER IERROR
C
      CALL MPI_COMM_SPLIT(COMM, COLOR, KEY, NEWCOMM, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Comm_Rank(COMM, RANK, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Gives the rank of the process  in the group COMM
C
      INTEGER COMM, RANK
      INTEGER IERROR
C
      CALL MPI_COMM_RANK(COMM, RANK, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Comm_Size(COMM, SIZE, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Gives the number of processes in the group COMM
C
      INTEGER COMM, SIZE
      INTEGER IERROR
C
      CALL MPI_COMM_SIZE(COMM, SIZE, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Type_Commit(DATATYPE, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Commits the DATATYPE
C
      INTEGER DATATYPE
      INTEGER IERROR
C
      CALL MPI_TYPE_COMMIT(DATATYPE, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Type_Free(DATATYPE, IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Sets the DATATYPE to MPI_DATATYPE_NULL
C
      INTEGER DATATYPE
      INTEGER IERROR
C
      CALL MPI_TYPE_FREE(DATATYPE, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Type_Vector(COUNT, BLEN, STRIDE, OLDTYPE, NEWTYPE,
     *                           IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Datatype constructor
C
      INTEGER COUNT, BLEN, STRIDE, OLDTYPE, NEWTYPE
      INTEGER IERROR
C
      CALL MPI_TYPE_VECTOR(COUNT, BLEN, STRIDE, OLDTYPE,
     *                     NEWTYPE, IERROR)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_Reduce(SBUF, RBUF, COUNT, DATATYPE, OP, ROOT, COMM,
     *                      IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Global functions, result is at ROOT
C
      REAL SBUF(*), RBUF(*)
      INTEGER COUNT, DATATYPE, OP, ROOT, COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
      INTEGER MYID
C
      MYID = MPX_MYID(MPX_COMM_WORLD, IERROR)
      T1 = MPX_WTime()
      CALL MPI_REDUCE(SBUF, RBUF, COUNT, DATATYPE, OP, ROOT,
     *                     COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      IF(MYID .NE. ROOT) THEN
         MPX_LENGTH = MPX_LENGTH + COUNT
         MPX_COUNT = MPX_COUNT + 1
      END IF
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_AllReduce(SBUF, RBUF, COUNT, DATATYPE, OP, COMM,
     *                         IERROR)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Global functions, result is at each process
C
      REAL SBUF(*), RBUF(*)
      INTEGER COUNT, DATATYPE, OP, COMM
      INTEGER IERROR
      DOUBLE PRECISION T1, T2
      INTEGER MYID
C
      MYID = MPX_MYID(MPX_COMM_WORLD, IERROR)
      T1 = MPX_WTime()
      CALL MPI_ALLREDUCE(SBUF, RBUF, COUNT, DATATYPE, OP,
     *                     COMM, IERROR)
      T2 = MPX_WTime()
      MPX_TIME = MPX_TIME + T2 - T1 - MPX_DT
      MPX_LENGTH = MPX_LENGTH + COUNT
      MPX_COUNT = MPX_COUNT + 1
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_EHANDLER()
C
C Predefined subroutine for error handling
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE MPX_STATISTICS(COUNT, LENGTH, TIME)
      INCLUDE 'mpif.h'
      INCLUDE 'mpxf.h'
C
C Returns run-time statistics
C
      INTEGER COUNT, LENGTH
      DOUBLE PRECISION TIME
C
      COUNT = MPX_COUNT
      LENGTH = MPX_LENGTH
      TIME = MPX_TIME
C
      RETURN
      END
C-----------------------------------------------------------------------
